@@ -31,11 +31,18 @@ class TestBasicCommandFunctionality(unittest.TestCase):
     test to verify basic CLI functionality isn't entirely broken.
     """
 
-    def put_object(self, bucket, key, content, extra_args=None):
+    def put_object(
+        self, 
+        bucket, 
+        key, 
+        content, 
+        extra_args=None, 
+    ):
         session = botocore.session.get_session()
         client = session.create_client('s3', 'us-east-1')
-        client.create_bucket(Bucket=bucket)
+        client.create_bucket(Bucket=bucket, ObjectOwnership='ObjectWriter')
         time.sleep(5)
+        client.delete_public_access_block(Bucket=bucket)
         self.addCleanup(client.delete_bucket, Bucket=bucket)
         call_args = {
             'Bucket': bucket,
